/*******
svmvia, an implementation of the full regularization path for support vector machines
Copyright (C) 2007 Oliver Serang
Written by Oliver Serang at the Department of Genome Sciences at the University of Washington

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*********/

#include "basic.h"
#include "seq.h"
#include "matrix.h"

double DEGREE, BIAS, GAMMA, LAMBDA_MIN;
bool CACHE;
int K_TYPE;
int ERR_MODE;
bool VERBOSE;
bool PRIMAL_HYPERPLANE;

matrix kernelMat;

double max(double a, double b)
{
  if (a > b)
    return a;
  return b;
}

double min(double a, double b)
{
  if (a < b)
    return a;
  return b;
}

int max(int a, int b)
{
  if (a > b)
    return a;
  return b;
}

int min(int a, int b)
{
  if (a < b)
    return a;
  return b;
}

double sign(double d)
{
  if (d>0) return 1;
  return -1;
}

void swap(double & a, double & b)
{
  double temp = a;
  a = b;
  b = temp;
}

double kFunction(const seq & lhs, const seq & rhs)
{
  //    return exp( - square( magnitude(lhs - rhs)));
  //  return exp( -1 * magDiffSquare(lhs, rhs));
  //    return dotProd(lhs, rhs);
  //      return pow(dotProd(lhs,rhs) + 1, 3);
  if (K_TYPE == 0)
    {
      return rbfKernel(lhs, rhs);
    }
  else
    {
      return polyKernel(lhs, rhs);
    }
}

double rbfKernel(const seq & lhs, const seq & rhs)
{
  return exp(-GAMMA * magDiffSquare(lhs, rhs));
}

double polyKernel(const seq & lhs, const seq & rhs)
{
  return pow(dotProd(lhs, rhs) + BIAS, DEGREE);
}

inline double square(double x)
{
  return x * x;
}

double randReal(double d)
{
  return d * (rand() % 100000) / 99999.0;
}
