/*******
svmvia, an implementation of the full regularization path for support vector machines
Copyright (C) 2007 Oliver Serang
Written by Oliver Serang at the Department of Genome Sciences at the University of Washington

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*********/

#include <iostream>
#include "runSVM.h"
#include "basic.h"

#include <fstream>

using namespace std;

char * trainFile, * valFile, * saveFile;

char * setFile(char * fname)
{
  ifstream fin(fname);
  if (fin.fail())
    {
      cerr << "usage: no file found named '" << fname << "'" << endl << endl;
      exit(1);
    }
  return fname;
}

double setFloat(char * p)
{
  float t;
  //  cout << "Set float: " << p << endl;
  int L = sscanf(p, "%f", &t);
  if (L == 0)
    {
      cerr << "usage: gamma, bias, and degree must be valid floating point numbers" << endl << endl;
      exit(1);
    }
  return double(t);
}

void setKernel(char * p)
{
  if (strcmp(p, "rbf") == 0)
    {
      K_TYPE = 0;
    }
  else if (strcmp(p, "poly") == 0)
    {
      K_TYPE = 1;
    }
  else
    {
      cerr << "usage: -K [kernel type]" << endl << endl;
      exit(1);
    }
}

void setErrMode(char * p)
{
  if (strcmp(p, "misclass") == 0)
    {
      ERR_MODE = 0;
    }
  else if (strcmp(p, "weighted") == 0)
    {
      ERR_MODE = 1;
    }
  else if (strcmp(p, "roc") == 0)
    {
      ERR_MODE = 2;
    }
  else
    {
      cerr << "usage: -errmode {misclass,weighted,roc}" << endl << endl;
      exit(1);
    }
}

void setVerbose(char * p)
{
  if (strcmp(p, "true") == 0)
    {
      VERBOSE = true;
    }
  else if (strcmp(p, "false") == 0)
    {
      VERBOSE = false;
    }
  else
    {
      cerr << "usage: -verbose {true,false}" << endl << endl;
      exit(1);
    }
}

void setCache(char * p)
{
  if (strcmp(p, "true") == 0)
    {
      CACHE = true;
    }
  else if (strcmp(p, "false") == 0)
    {
      CACHE = false;
    }
  else
    {
      cerr << "usage: -cache {true,false}" << endl << endl;
      exit(1);
    }
}

void setOpts(int argc, char * argv[])
{
  int k;
  for (k=1; k<argc; k++)
    {
      if ( strcmp(argv[k], "-K") == 0)
	{
	  setKernel(argv[k+1]);
	  break;
	}
    }
  if (k == argc)
    {
      cerr << "Kernel type not specified, using RBF" << endl;
      K_TYPE = 0;
    }

  if (K_TYPE == 0)
    {
  for (k=1; k<argc; k++)
    {
      if ( strcmp(argv[k], "-gamma") == 0)
	{
	  GAMMA = setFloat(argv[k+1]);
	  break;
	}
    }
  if (k == argc)
    {
      cerr << "gamma not specified, using 1" << endl;
      GAMMA = 1;
    }
    }
  else if (K_TYPE == 1)
    {
  for (k=1; k<argc; k++)
    {
      if ( strcmp(argv[k], "-bias") == 0)
	{
	  BIAS = setFloat(argv[k+1]);
	  break;
	}
    }
  if (k == argc)
    {
      cerr << "bias not specified, using 1" << endl;
      BIAS = 1;
    }

  for (k=1; k<argc; k++)
    {
      if ( strcmp(argv[k], "-degree") == 0)
	{
	  DEGREE = setFloat(argv[k+1]);
	  break;
	}
    }
  if (k == argc)
    {
      cerr << "degree not specified, using 1" << endl;
      DEGREE = 1;
    }
    }


    for (k=1; k<argc; k++)
    {
      if ( strcmp(argv[k], "-cache") == 0)
	{
	  setCache(argv[k+1]);
	  break;
	}
    }
  if (k == argc)
    {
      cerr << "cache not specified, not caching" << endl;
      CACHE = false;
    }


    for (k=1; k<argc; k++)
    {
      if ( strcmp(argv[k], "-verbose") == 0)
	{
	  setVerbose(argv[k+1]);
	  break;
	}
    }
  if (k == argc)
    {
      cerr << "verbosity not specified, running silent" << endl;
      VERBOSE = false;
    }


    for (k=1; k<argc; k++)
    {
      if ( strcmp(argv[k], "-errmode") == 0)
	{
	  setErrMode(argv[k+1]);
	  break;
	}
    }
  if (k == argc)
    {
      cerr << "errmode not specified, using misclass" << endl;
      ERR_MODE = 0;
    }



    for (k=1; k<argc; k++)
    {
      if ( strcmp(argv[k], "-train") == 0)
	{
	  trainFile = setFile(argv[k+1]);
	  break;
	}
    }
  if (k == argc)
    {
      cerr << "usage: no training file specified" << endl;
      exit(1);
    }


    for (k=1; k<argc; k++)
    {
      if ( strcmp(argv[k], "-val") == 0)
	{
	  valFile = setFile(argv[k+1]);
	  break;
	}
    }
  if (k == argc)
    {
      cerr << "usage: no validation file specified" << endl;
      exit(1);
    }



    for (k=1; k<argc; k++)
    {
      if ( strcmp(argv[k], "-model") == 0)
	{
	  if (strcmp(argv[k+1], "") == 0)
	    {
	      //	      cout << "Try " << argv[k+1] << endl;
	      cerr << "usage: valid model file to save to not specified" << endl << endl;
	      exit(1);
	    }
	  else
	    saveFile = argv[k+1];
	    break;
	}
    }
  if (k == argc)
    {
      cerr << "usage: no model file specified to save to" << endl;
      exit(1);
    }

    for (k=1; k<argc; k++)
    {
      if ( strcmp(argv[k], "-maxC") == 0)
	{
	  LAMBDA_MIN = 1 / setFloat(argv[k+1]);
	  break;
	}
    }
  if (k == argc)
    {
      cerr << "no maximum C value specified, using 10000" << endl;
      LAMBDA_MIN = 1e-4;
    }

    for (k=1; k<argc; k++)
    {
      if ( strcmp(argv[k], "-primal-hyperplane") == 0)
	{
	  if ( K_TYPE == 0 )
	    {
	      cerr << "error: primal-hyperplane not supported on RBF kernel" << endl << endl;
	      exit(1);
	    }
	  if ( DEGREE != 1 )
	    {
	      cerr << "error: primal-hyperplane only supported with degree 1" << endl << endl;
	      exit(1);
	    }

	  PRIMAL_HYPERPLANE = true;
	  break;
	}
    }
  if (k == argc)
    {
      PRIMAL_HYPERPLANE = false;
    }


  // -K {rbf, poly}
  // -bias -degree
  // -gamma
  //
  // -cache {true, false}
  // data
  // validation data
}

int main (int argc, char * argv[])
{
        srand(1);
  //  srand(time(NULL));

  char ** argvBuffer = new char *[argc+1];
  int k;
  for (k=0; k<argc; k++)
    argvBuffer[k] = argv[k];

  argvBuffer[k] = "";

  setOpts(argc, argvBuffer);
  
  //  cout << "K_TYPE = " << K_TYPE << endl;

  //  cout << "Gamma : " << GAMMA << endl;

  //  ValSVM meta("./genData", 100, 2);
  //  meta.run();

  runSVM thinker(trainFile, valFile);
  thinker.run();
  thinker.saveSVM(saveFile);

  return 0;
}

