/*******
svmvia, an implementation of the full regularization path for support vector machines
Copyright (C) 2007 Oliver Serang
Written by Oliver Serang at the Department of Genome Sciences at the University of Washington

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*********/

#ifndef MATRIX_H_
#define MATRIX_H_

using namespace std;

#include <iostream>
#include <vector>

#include "seq.h"
#include "set.h"

struct matrix
{
public:
	matrix();
	matrix(int rr, int cc);
	vector<seq> data;

	int r;
	int c;
	seq & operator [](int k);
  seq operator [] (int k) const;
  matrix operator [] (const set & rhs);

	const matrix & operator *=(matrix & rhs);
	const matrix & operator +=(matrix & rhs);
	const matrix & operator *=(double rhs);

	matrix operator +(matrix & rhs);
	matrix operator *(matrix & rhs);

  void resize(int rr, int cc);

  void removeRC(int i);

  seq operator *(const seq & rhs);

	matrix operator *(double rhs);
	matrix operator -();

  matrix restrictCol(const set & rhs);

	void invert();
	matrix id(int R);
};

ostream & operator << (ostream & os, matrix m);

#endif
