/*******
svmvia, an implementation of the full regularization path for support vector machines
Copyright (C) 2007 Oliver Serang
Written by Oliver Serang at the Department of Genome Sciences at the University of Washington

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*********/

#include "runSVM.h"

runSVM::runSVM(char * fname, char * vname)
{
  ifstream fin(fname);
  ifstream fin_val(vname);
  
  fin >> n;
  fin_val >> n_val;
  fin >> x_dim;
  fin_val >> x_dim_val;

  if (x_dim != x_dim_val)
    {
      cerr << "error in data: dimension of x does not match in training and validation files" << endl << endl;
      exit(1);
    }

  x = matrix(n, x_dim);
  y = seq(n);

  x_val = matrix(n_val, x_dim_val);
  y_val = seq(n_val);

  int temp;
  char tempChar;

  int k,j;

  // read in data and then val data

  double value;

  for (k=0; k<n; k++)
    {
      for (j=0; j<x_dim; j++)
	x[k][j] = 0;
    }
  for (k=0; k<n_val; k++)
    {
      for (j=0; j<x_dim_val; j++)
	x_val[k][j] = 0;
    } 
 for (k=0; k<n; k++)
    {
		  fin >> y[k];
		  for (int j=0; j<x_dim; j++)
			{
			  fin >> temp >> tempChar;
			  fin >> value;
			  x[k][temp] = value;
			}
    }

  for (k=0; k<n_val; k++)
    {
		  fin_val >> y_val[k];
		  for (int j=0; j<x_dim_val; j++)
			{
			  fin_val >> temp >> tempChar;
			  fin_val >> value;
			  x_val[k][temp] = value;
			}
    }

  // NOTE: only works for x == valX
  // for gathering runtimes ONLY
  //    normalize(x);   
  //    normalize(x_val);
}

void runSVM::normalize(matrix & lhs)
{
  int k, j;
  for (k=0; k<lhs.c; k++)
    {
      double xMax = -infinity, xMin = infinity;
      for (j=0; j<lhs.r; j++)
	{
	  if (lhs[j][k] > xMax)
	    xMax = lhs[j][k];
	  if (lhs[j][k] < xMin)
            xMin = lhs[j][k];
	}
      for (j=0; j<lhs.r; j++)
	{
	  lhs[j][k] = (lhs[j][k] - xMin)/(xMax-xMin) * 2 - 1;
	}
    }
}

double runSVM::run()
{
  //  cout << x << endl << y << endl;
  //  cout << x_val << endl << y_val << endl;

  //  hastie = hastieSVM(x, y, x_val, y_val);
  hastie = SVM(x, y, x_val, y_val);
  hastie.fitPath();
  cout << "Validation error = " << hastie.bestError << endl;
  cout << "Optimal C = " << 1/hastie.lambdaOpt << endl;
}

void runSVM::saveSVM(char * fname)
{
  cout << "Saving model to the file '" << fname << "'" << endl;
  ofstream fout(fname);

  fout << K_TYPE << endl;
  if (K_TYPE == 0)
    fout << GAMMA << endl;
  else
    fout << BIAS << " " << DEGREE << endl;
  fout << CACHE << endl;

  fout << n << " " << x_dim << endl;
  int k, j;
  fout << hastie.lambdaOpt << endl;
  for (k=0; k<n; k++)
    {
      fout << hastie.alphaOpt[k] << "\t";
      fout << y[k] << " ";
      for (j=0; j < x_dim; j++)
	{
	  fout << x[k][j] << " ";
	}
      fout << endl;
    }
  fout << hastie.alpha0Opt << endl;

  if ( PRIMAL_HYPERPLANE )
    {
      for (j=0; j<x_dim; j++)
	{
	  double tot = 0;
	  for (k=0; k<n; k++)
	    {
	      tot += y[k] * hastie.alphaOpt[k] * x[k][j];
	    }
	  fout << tot << " ";
	}
      fout << endl;
    }
}

void runSVM::loadSVM(char * fname)
{
  cout << "Loading svm" << endl;
  ifstream fin(fname);

  fin >> K_TYPE;
  if (K_TYPE == 0)
    fin >> GAMMA;
  else
    fin >> BIAS >> DEGREE;
  fin >> CACHE;


  fin >> n >> x_dim;

  hastie.n = n;

  hastie.x = matrix(n, x_dim);
  hastie.y = seq(n);

  hastie.alphaOpt = seq(n);

  int k, j;
  fin >> hastie.lambdaOpt;
  for (k=0; k<n; k++)
    {
      fin >> hastie.alphaOpt[k];
      fin >> hastie.y[k];
      for (j=0; j < x_dim; j++)
	{
	  fin >> hastie.x[k][j];
	}
    }
  fin >> hastie.alpha0Opt;
  //  cout << "SVM loaded..." << endl;
}

void runSVM::predict(char * fname, char * oname)
{
  ifstream fin(fname);

  int n_samp, x_dim_samp;

  fin >> n_samp >> x_dim_samp;

  if (x_dim != x_dim_samp)
    {
      cerr << "error in data: dimension of x does not match in predicted file" << endl << endl;
      exit(1);
    }

  seq x_samp(x_dim);
  double y_samp;

  int temp;
  char tempChar;

  int k,j;

  ofstream fout(oname);

  double value;
  int cor = 0;
  for (k=0; k<n_samp; k++)
    {
		  fin >> y_samp;
		  for (int j=0; j<x_dim; j++)
			{
			  fin >> temp >> tempChar;
			  fin >> value;
			  x_samp[j] = value;
			}
		  double disc = hastie.deNovoOpt(x_samp);
		  double pred = sign(disc);
		  fout << setw(10) << disc << '\t';
		  fout << pred << endl;
		  if (pred == y_samp)
		    cor++;
    }
  cout << "Predicted " << double(cor) / n_samp << " correctly" << endl;
}

