/*******
svmvia, an implementation of the full regularization path for support vector machines
Copyright (C) 2007 Oliver Serang
Written by Oliver Serang at the Department of Genome Sciences at the University of Washington

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*********/

#ifndef SEQ_H_
#define SEQ_H_

#include <vector>
#include <iostream>
#include <math.h>

#include "set.h"
#include "basic.h"

using namespace std;

class seq
{
 public:
  seq() {}
  seq(const seq & cp):data(cp.data) {}
  seq(int sz):data(sz) {}

    void check_size(const seq & rhs);

    void error(char *);

  vector<double> data;
  const seq & operator +=(double rhs);
  const seq & operator -=(double rhs);
  const seq & operator *=(double rhs);
  const seq & operator /=(double rhs);

  seq operator +(double rhs) const;
  seq operator -(double rhs) const;
  seq operator *(double rhs) const;
  seq operator /(double rhs) const;

  const seq & operator +=(const seq & rhs);
  const seq & operator -=(const seq & rhs);
  const seq & operator *=(const seq & rhs);
  const seq & operator /=(const seq & rhs);

  seq operator +(const seq & rhs) const;
  seq operator -(const seq & rhs) const;
  seq operator *(const seq & rhs) const;
  seq operator /(const seq & rhs) const;
  seq operator -() const;

  void resize(int s);

  int size() const;
  double & operator[] (int k);
  double operator [] (int k) const
  {
    return data[k];
  }
  seq operator[] (const set & rhs);
  set match(double val) const;
  int numMatch(double val) const;

  set maxSet();
  set minSet();

  double max();
  double min();

  int search(double d);

  void join(const seq & rhs);

  void setAll(double d);
  set operator < (double d);
  set operator >(double d);
};

ostream & operator << (ostream & os, seq rhs);

void swap(seq & x, seq & y);

double sum(const seq & rhs);
double absSum(const seq & rhs);
double dotProd(const seq & lhs, const seq & rhs);

double max(seq rhs);
double min(seq rhs);

double magnitude(const seq & rhs);

double magDiffSquare(const seq & lhs, const seq & rhs);

/***
double magDiffSquare(const seq & lhs, const seq & rhs)
{
  double tot = 0;
  for (int k=0; k<lhs.size(); k++)
    {
      tot += (lhs[k] - rhs[k]) * (lhs[k] - rhs[k]);
    }
  return tot;
}
***/

seq abs(const seq & rhs);

#endif
