/*******
svmvia, an implementation of the full regularization path for support vector machines
Copyright (C) 2007 Oliver Serang
Written by Oliver Serang at the Department of Genome Sciences at the University of Washington

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*********/

#include "set.h"

set::set(int s):
  data(s)
{
  // initialize the set to be full
  for (int k=0; k<size(); k++)
    {
      data[k] = k;
    }
}

int set::maxIndex() const
{
  int maxI = -1;
  for (int k=0; k<size(); k++)
    {
      if (data[k] > maxI)
	maxI = data[k];
    }
  return maxI;
}

int & set::operator [](int k)
{
  return data[k];
}

int set::operator [](int k) const
{
  return data[k];
}

set set::operator [](const set & rhs) const
{
  set ret(rhs.size());
  int k;
  for (k=0; k<ret.size(); k++)
    {
      ret[k] = (*this)[ rhs[k] ];
    }
  return ret;
}

void set::fill(int cap)
{
  data = vector<int>(cap);
  for (int k=0; k<cap; k++)
    {
      data[k] = k;
    }
}

void set::clear()
{
  data = vector<int>(0);
}

void set::join(const set & rhs)
{
  // you can code this later with bitsets to be better

  // the vector should be sized to match the largest
  // element in *this and rhs
  int k;

  int cap = maxIndex() + 1;
  int cap2 = rhs.maxIndex() + 1;
  if (cap2 > cap)
    cap = cap2;

  vector<int> temp(cap);

  // clear the initial table
  for (k=0; k<cap; k++)
    {
      temp[k] = 0;
    }

  // add all of the elements in the current set
  for (k=0; k<size(); k++)
    {
      temp[ (*this)[k] ] = 1;
    }

  // add members of rhs
  for (k=0; k<rhs.size(); k++)
    {
      temp[ rhs[k] ] = 1;
    }

  // count the non-zero elements
  int count = 0;
  for (k=0; k<cap; k++)
    {
      if (temp[k] == 1)
	count++;
    }

  data.resize(count);

  int c = 0;
  for (k=0; k<cap; k++)
    {
      if (temp[k] == 1)
	{
	  data[c] = k;
	  c++;
	}
    }
}

void set::remove(const set & rhs)
{
  // you can code this later with bitsets to be better

  // the vector should be sized to match the largest
  // element in *this
  int k;
  int cap = maxIndex() + 1;
  vector<int> temp(cap);

  // clear the initial table
  for (k=0; k<cap; k++)
    {
      temp[k] = 0;
    }

  // add all of the elements in the current set
  for (k=0; k<size(); k++)
    {
      temp[ (*this)[k] ] = 1;
    }

  // remove the elements in the valid range that are
  // members of rhs
  for (k=0; k<rhs.size(); k++)
    {
      if (rhs[k] < cap)
	{
	  temp[ rhs[k] ] = 0;
	}
    }

  // count the non-zero elements
  int count = 0;
  for (k=0; k<cap; k++)
    {
      if (temp[k] == 1)
	count++;
    }

  data.resize(count);
  int c = 0;
  for (k=0; k<cap; k++)
    {
      if (temp[k] == 1)
	{
	  data[c] = k;
	  c++;
	}
    }
}


void set::remove(int i)
{
  //cout << "REMOVE INT FROM SET" << endl;
  int k;
  int index = -1;
  for (k=0; k<size(); k++)
    {
      if (data[k] == i)
	  {
		index = k;
		break;
	  }
    }
  if (index == -1)
    {
      // data not found - don't bother
    }
  else
    {
		removeIndex(index);
    }
}

void set::removeIndex(int index)
{
// i should be in the size...

	int k;
	for (k=index; k<(size()-1); k++)
	{
	  data[k] = data[k+1];
	}
	data.resize(size()-1);
}

int set::search(int i)
{
  for (int k=0; k<size(); k++)
    if (data[k] == i)
      return k;
  return -1;
}

  void set::add(int i)
    {
      data.resize(size() + 1);
      data[size()-1] = i;
    }

bool set::contains(int i)
{
  for (int k=0; k<size(); k++)
    {
      if (data[k] == i)
	return true;
    }
  return false;
}

ostream & operator << (ostream & os, set rhs)
{
  int w = os.width();
  os << "{ ";
  for (int k=0; k<rhs.size(); k++)
    {
      os.width(5);
      os << rhs[k];
      if (k != rhs.size()-1)
	os << ", ";
    }
  os << " }";
  os.width(w);
  return os;
}


