/*******
	svmvia, an implementation of the full regularization path for support vector machines
	Copyright (C) 2007 Oliver Serang
	Written by Oliver Serang at the Department of Genome Sciences at the University of Washington

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*********/

#include "smo.h"

SMO::SMO(matrix & xPar, seq & yPar):
  x(xPar), y(yPar), alpha(xPar.r), fC(xPar.r), kDiag(xPar.r)
{
  //	  cout << "Makine smo..." << endl << x << endl << y << endl;


  C = 1.0;

  int k, j;
  for (k=0; k<x.r; k++)
    {
      kDiag[k] = kFunction(x[k],x[k]);
    }

  /***
      K = matrix(x.r, x.r);
      for (k=0; k<K.r; k++)
      {
      for (j=0; j<K.r; j++)
      {
      K[k][j] = kFunction( x[k], x[j]);
      }
      }
      //		cout << "K DUMP: " << endl;
      //		cout << K << endl << endl << endl;
      ***/

		
  nMinus = 0;
  nPlus = 0;
  for (k=0; k<alpha.size(); k++)
    {
      if (y[k] < 0)
	{
	  nMinus++;
	}
      else
	{
	  nPlus++;
	}
    }

  if (nMinus < nPlus)
    {
      minor = -1;
      nMinor = nMinus;
    }
  else
    {
      minor = 1;
      nMinor = nPlus;
    }

  /***
  cout << "N- " << nMinus << endl;
  cout << "N+ " << nPlus << endl << endl;
  cout << "minor is: " << minor << endl;
  ***/

  int count = 0;
  for (k=0; k<alpha.size(); k++)
    {
      if (y[k] == minor)
	{
	  alpha[k] = 1;
	}
      else
	{
	  if (count < nMinor)
	    {
	      alpha[k] = 1;
	      count ++;
	    }
	}
    }

  /****
  for (k=0; k<alpha.size(); k++)
    {
      if (y[k] != minor)
	{
	  alpha[k] = double(nMinor) / (nPlus + nMinus - nMinor);
	}
    }
  ****/

  //  minor = 0;

  double T = 0;
  for (k=0; k<alpha.size(); k++)
    {
      T += alpha[k] * y[k];
    }
  //  cout << "Initial Sum[y[i]*alpha[i],i]\t" << T << endl;

  beta = 0;

  for (k=0; k<alpha.size(); k++)
    {
      fC[k] = f(k) - beta;
    }

  updateBeta();
  initObjective();

  int c = 0;
  for (k=0; k<y.size(); k++)
    {
      if ( y[k] != minor )
	c++;
    }
  //  cout << "Major = " << c << endl;

  //		cout << "Initial beta: " << beta << endl;


  //		cout << "The initial state is:" << endl;
  //		for (k=0; k<alpha.size(); k++)
  //		  cout << alpha[k] << " ";
  //		cout << endl;
  //		cout << beta << endl << endl;
}

void SMO::printModel()
{
  int k;
  cout << "alpha = ";
  for (k=0; k<alpha.size(); k++)
    {
      cout << alpha[k] << " ";
    }
  cout << endl;
  cout << "W = ";
  for (k=0; k<x.c; k++)
    {
      double tot = 0;
      for (int j=0; j<x.r; j++)
	{
	  tot += alpha[j] * y[j] * x[j][k];
	}
      cout << tot << " ";
    }
  cout << endl << "BETA: " << beta << endl;
  cout << "lambda: " << lambda << endl << endl;
}

void SMO::createSets(set & left, set & elbow, set & right)
{
  left.clear();
  elbow.clear();
  right.clear();
  for (int k=0; k<x.r; k++)
    {
      if (fLambda(k)*y[k] - 1 > epsilon)
	{
	  // correct
	  //		cout << "Right: " << k << endl;
	  right.add(k);
	}
      else if (fabs(fLambda(k) * y[k] - 1) < epsilon)
	{
	  // margin
	  //		cout << "Margin: " << k << endl;
	  elbow.add(k);
	}
      else
	{
	  // incorrect
	  //		cout << "Incorrect: " << k << endl;
	  left.add(k);
	}
    }
}

int SMO::takeStep(int i, int j)
{
  if (y[i] == minor || y[j] == minor)
    return 0;

  if (i==j)
    return 0;

  int k, l;
  //	cout << "\tAttempting pair " << i << " " << j << endl;
  //	double gamma = 
  double s = y[i] * y[j];
	
  double Ei = fCache(i) - y[i];
  double Ej = fCache(j) - y[j];
	
  double L, H;
  if (s != 1)
    {
      L = max(0.0, alpha[i] - alpha[j]);
      H = min(C, C - alpha[j] + alpha[i]);
    }
  else
    {
      L = max(0.0, alpha[j] + alpha[i] - C);
      H = min(C, alpha[j] + alpha[i]);
    }
  if (L==H)
    {
      //	    	    cout << "No pass because L == H" << endl;
      return 0;
    }

  double kappa = kernel(i,i) + kernel(j,j) - 2*kernel(j,i);
	
  double alphaI;
  if (kappa > 0)
    {
      alphaI = alpha[i] + y[i]*(Ej - Ei)/kappa;
      // clip alphaJ...
      alphaI = min(max(alphaI, L), H);
    }
  else if (y[i] * (Ej - Ei) < 0)
    {
      alphaI = H;
    }
  else
    alphaI = L;

  if (fabs(alphaI - alpha[i]) < epsilon * (alpha[i]+alphaI+epsilon))
    {
      //	  	  cout << "No pass because of small step" << endl;
      return 0;
    }

  double alphaJ = alpha[j] + s*(alpha[i] - alphaI);

  for (k=0; k<fC.size(); k++)
    {
      fC[k] += y[i]*kernel(k,i)*(alphaI-alpha[i]) + y[j]*kernel(k,j)*(alphaJ-alpha[j]);
    }

  double oldObj = obj;

  //	cout << "Chg " << i <<  " " << alpha[i] << " -> " << alphaI << endl;
  //	cout << "Chg " << j <<  " " << alpha[j] << " -> " << alphaJ << endl << endl;

  //	initObjective();
  //	cout << "Starting obj = " << obj << endl;
  double pseudoObj = obj;

  /***
      for (k=0; k<x.r; k++)
      {
      if ( k != i && k != j )
      pseudoObj += alpha[k];
      }
  ***/

  pseudoObj += alphaI - alpha[i];
  pseudoObj += alphaJ - alpha[j];

  /***
      for (k=0; k<x.r; k++)
      {
      for (l=0; l<x.r; l++)
      {
      if ( k != i && l != i && k != j && l != j )
      {
      pseudoObj -= .5 * alpha[k] * alpha[l] * y[k] * y[l] * kernel(k,l);
      }
      }
      }
  ***/

  for (k=0; k<x.r; k++)
    {
      if ( k != i && k != j )
	{
	  pseudoObj -= alpha[k] * (alphaI - alpha[i]) * y[k] * y[i] * kernel(k, i);
	  pseudoObj -= alpha[k] * (alphaJ - alpha[j]) * y[k] * y[j] * kernel(k, j);
	}
    }

  //	pseudoObj -= (alphaI - alpha[i]) * (alphaJ - alpha[j]) * y[i] * y[j] * kernel(i,j);
  pseudoObj += alpha[i] * alpha[j] * y[i] * y[j] * kernel(i,j);
  pseudoObj -= alphaI * alphaJ * y[i] * y[j] * kernel(i,j);

  pseudoObj += .5 * alpha[i] * alpha[i] * y[i] * y[i] * kernel(i,i);
  pseudoObj -= .5 * alphaI * alphaI * y[i] * y[i] * kernel(i,i);

  pseudoObj += .5 * alpha[j] * alpha[j] * y[j] * y[j] * kernel(j,j);
  pseudoObj -= .5 * alphaJ * alphaJ * y[j] * y[j] * kernel(j,j);

  //	cout << "Pseudo obj: " << pseudoObj << endl;

  alpha[j] = alphaJ;
  alpha[i] = alphaI;

  //  nonBoundSV.checkIntegrity();


  //	initObjective();
  //	cout << "True obj  : " << obj << endl;
	
  updateBeta();

  if ( fabs(obj - pseudoObj) < epsilon )
    return 0;

  obj = pseudoObj;

  //	cout << "\tSuccess!" << endl;
  return 1;
}

void SMO::updateBeta()
{
  int k;
  double pos = -infinity, neg = -infinity;

  int index = -1;
  int indexB = -1;
  for (k=0; k<x.r; k++)
    {
      // rewrite later for epsilon...
      //		if (y[k] != minor && alpha[k] > epsilon)
      if (y[k] != minor && alpha[k] > epsilon)
	{
	  if (pos < y[k] * (fCache(k) - beta))
	    {
	      pos = y[k] * (fCache(k) - beta);
	      index = k;
	    }
	}
      if ( y[k] == minor)
	{
	  if (neg < y[k] * (fCache(k) - beta))
	    {
	      neg = y[k] * (fCache(k) - beta);
	      indexB = k;
	    }
	}
    }
  //	cout << "Index: " << index << ", minor Index: " << indexB << endl;
  beta = (pos - neg)/-2.0 * -minor;

  // find the max value...
  int ind = -1;
  double indV = -infinity;
  for(k=0; k<x.r; k++)
    {
      if ( alpha[k] > epsilon && fCache(k) * y[k] > indV)
	{
	  indV = fCache(k)*y[k];
	  ind = k;
	  //		cout << "Trading to new lambda: " << indV << " " << index << endl;
	}
    }

  lambda = indV;
  /***
      for (k=0; k<x.r; k++)
      {
      if (fLambda(k) * y[k] > 1 && alpha[k] > epsilon || fLambda(k) * y[k] < 1 && alpha[k] < epsilon)
      {
      //		cout << "culprit: it were sport uncle " << k << endl;
      cout << fLambda(k) * y[k] << " " << alpha[k] << endl;
      getchar();
      }
      }

      //	cout << "WTF, mates: " << fCache(index) << endl;
      if ( fabs( fCache(index) + fCache(indexB)) > .001)
      {
      cout << "Diff f: " << index << " " << indexB << " -> " << fabs( fCache(index) + fCache(indexB)) << endl;
      }
  ***/

  //	cout << "New beta = " << beta << endl;
}

bool SMO::notBound(double d)
{
  return d > (epsilon) && d < (C-epsilon);
}

void SMO::buildNonZeroBound()
{
  int k;
  int c = 0;
  nonBoundSV.clear();
  for ( k=0; k<alpha.size(); k++)
    {
      if ( y[k] != minor && notBound(alpha[k]))
	{
	  nonBoundSV.push_back(k);
	  c++;
	}
    }
  //  cout << "Rebuilt size is: " << c << endl;
}

bool SMO::violates(int i)
{
  double E = fCache(i) - y[i];
  double r = E * y[i];

  // good code
  //    if ((r < -TOL && alpha[i] < (C - epsilon) ) || (r>TOL && alpha[i] > epsilon))
  if ((r < -TOL && alpha[i] < (C) ) || (r>TOL && alpha[i] > 0))
    {
      //      cout << "pt " << i << " violates " << r << " " << alpha[i] << endl;
      return true;
    }
  return false;
}

bool SMO::violatesLambda(int i)
{
  double E = fLambda(i) - y[i];
  double r = E * y[i];
  if ((r < -TOL && alpha[i] < (C-epsilon)) || (r>TOL && alpha[i] > epsilon))
    {
      return true;
    }
  return false;
}

int SMO::examineExample(int i)
{
  //  cout << "Examine " << i << endl;
  double E = fCache(i) - y[i];
  double r = E * y[i];
  //  if ((r < -TOL && alpha[i] < C) | (r>TOL && alpha[i] > 0))

  if (violates(i) && y[i] != minor)
    {
      int k;
      //	cout << "Trying largest margin" << endl;

      if ( ! nonBoundSV.empty() )
	{
	  k = secondChoice(i);
	  if (takeStep(i,k) == 1)
	    {
	      return 1;
	    }
	}

      //		cout << "Trying non bound alpha" << endl;
      /***
      int start = rand() % alpha.size();
      for (k=0; k<alpha.size(); k++)
	{
	  int p = (k+start) % alpha.size();
	  if ( notBound(alpha[p]) && y[p] != minor)
	    {
	      if (takeStep(i, p) == 1)
		{
		  return 1;
		}
	    }
	}
      ***/

      for (list<int>::iterator iter = nonBoundSV.begin(); iter != nonBoundSV.end(); iter++)
	{
	  if ( takeStep(i, *iter) )
	    return 1;
 	}

      //		cout << "Trying all other alpha" << endl;
      int start = rand() % alpha.size();
      for (k=0; k<alpha.size(); k++)
	{
	  if (y[k] != minor)
	    {
	      if (takeStep(i, (k+start) % alpha.size() ) == 1)
		{
		  return 1;
		}
	    }
	}
      //		cout << "Point " << i << " is violate, but doesn't pass..." << endl;
    }
  else
    {
      //	cout << "\tNo violation" << endl;
    }
  return 0;
}

int SMO::secondChoice(int i)
{
  int k,j;
  int ret = -1;
  int start = rand() % alpha.size();
  for (j=0; j<x.r; j++)
    {
      int k = (start + j) % alpha.size();
      if ( y[k] != minor)
	if ( ret == -1 || fabs( fCache(k) - y[k] - (fCache(i) - y[i])) > fabs( fCache(ret) - y[ret] - (fCache(i) - y[i])) )
	  ret = k;
    }

  
  /***
  for (list<int>::iterator iter = nonBoundSV.begin(); iter != nonBoundSV.end(); iter++)
    {
      k = *iter;
	if ( ret == -1 || fabs( fCache(k) - y[k] - (fCache(i) - y[i])) > fabs( fCache(ret) - y[ret] - (fCache(i) - y[i])) )
	  ret = k;
    }
  ***/

  return ret;
}

void SMO::initObjective()
{
  obj = 0;
  int k,j;
  for (k=0; k<x.r; k++)
    {
      obj += alpha[k];
    }

  for (k=0; k<x.r; k++)
    {
      for (j=0; j< x.r; j++)
	{
	  obj -= .5 * (y[k] * y[j] * alpha[k] * alpha[j] * kernel(k, j) );
	}
    }
}

void SMO::verify()
{
  /***
  // check the constraints and the objective...
  initObjective();
  cout << "Objective (scratch) = " << obj << endl;
  
  double sum = 0;
  for (int k=0; k<alpha.size(); k++)
    {
      if ( alpha[k] < 0 )
	cout << "Warning: small " << alpha[k] << endl;
      if ( alpha[k] > C )
	cout << "Warning: large " << alpha[k] << endl;
      sum += alpha[k] * y[k];
    }
  cout << "Sum [alpha * y] " << sum << endl;
  ***/
}

void SMO::simpleSMO()
{
  int k;
  int examineAll = 1;

  //  cout << "Obj starts at: " << obj << endl;
	
  int numChanged = 1;
  cycle = 0;
  while (numChanged > 0 || examineAll == 1 )
    {
      buildNonZeroBound();
      cycle++;

      if ( cycle % 100 == 0 )
	{
	  verify();
	}

      numChanged = 0;
      int numViolates = 0;

      if ( examineAll == 1 )
	{
	  examineAll = 0;
	  for (k=0; k<alpha.size(); k++)
	    {
	      if (y[k] != minor && violates(k))
		{
		  numChanged += examineExample(k);
		}
	    }
	}
      else
	{
	  for (list<int>::iterator iter = nonBoundSV.begin(); iter != nonBoundSV.end(); iter++)
	    {
	      numChanged += examineExample( *iter );
	    }
	  if (numChanged == 0)
	    examineAll = 1;
	}

      if ( VERBOSE )
	{
	  cout << cycle << " " << obj << endl;
	}

      /***
      cout << "Simple smo, cycle " << cycle << " changed " << numChanged << endl;
      cout << "Total violations for that round: " << numViolates << endl;
      cout << "Obj: " << obj << endl;
      ***/
    }
  //	printModel();

  //	cout << "Lambda final: " << lambda << endl;

  // code to check for fLambda violations- works but isn't informative
  /****
       for (k=0; k<alpha.size(); k++)
       {
       if (violatesLambda(k))
       {
       cout << "Warning: violations (lambda) reached after convergence in SMO..." << endl;
       if ( ! violates(k))
       {
       cout << "This is strange- no std violation..." << endl;
       }
       cout << "Beta = " << beta << endl;
       cout << "Point " << k << endl;
       cout << fLambda(k)*y[k] << "\t" << alpha[k] << endl;
       cout << fCache(k)*y[k] << endl;

       cout << "alpha:" << endl;
       for (int U = 0; U < alpha.size(); U++)
       cout << alpha[U] << " ";
       cout << endl;

       getchar();
       //		break;
       }
       }
       cout << endl;
  ***/

  /*****
	double T = 0;
	for (k=0; k<alpha.size(); k++)
	{
	T += (fCache(k) - beta);
	}
	cout << "\tThe initial lambda chosen in SMO was: " << lambda << endl;
	cout << "with beta = " << beta << endl;
	cout << "The ave. func value without beta = " << T/alpha.size() << endl;
	
	double aSum = 0;
	double aMax = -infinity;
	for (k=0; k<alpha.size(); k++)
	{
	aSum += alpha[k];
	if (alpha[k] > aMax)
	aMax = alpha[k];
	}
	cout << "Sum(alpha) = " << aSum << endl;
	cout << "Max alpha = " << aMax << endl;
	
	double kLarge = -infinity;
	int j;
	int bigK, bigJ;
	for (k=0; k<x.r; k++)
	{
	for (j=k; j<x.r; j++)
	{
	double t = kernel(k,j);
	if (t > kLarge)
	{
	kLarge = t;
	bigK = k;
	bigJ = j;
	}
	}
	}
	cout << "Largest kernel cell = " << kLarge << endl;
	cout << "\t(this happens in cell " << bigK << " " << bigJ << ") " << endl;
	cout << "(vectors are:) " << endl;
	cout << x[bigK] << endl;
	cout << x[bigJ] << endl;
  *****/

  //	cout << "Minor: " << minor << endl;

  cout << "Finished solving inital QP" << endl << endl;

}

double SMO::f(int p)
{
  double tot = 0;
  for (int k=0; k<alpha.size(); k++)
    {
      tot += y[k] * alpha[k] * kernel(p,k);
    }
  return tot + beta;
}

double SMO::fLambda(int p)
{
  return fCache(p) / lambda;
}

double SMO::fCache(int p)
{
  //    return f(p);
  return fC[p] + beta;
}


