/*******
	svmvia, an implementation of the full regularization path for support vector machines
	Copyright (C) 2007 Oliver Serang
	Written by Oliver Serang at the Department of Genome Sciences at the University of Washington

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*********/

#ifndef SMO_H_
#define SMO_H_

#include <iostream>
#include <vector>
#include <math.h>
#include "set.h"
#include "matrix.h"
#include "seq.h"
#include "basic.h"
#include <list>


using namespace std;

//#define C (1/0.705765)
//#define C 1.0

class SMO
{
 public:
  double C;


  void initObjective();
  double obj;

  int takeStep(int i, int j);
  int examineExample(int i);
  int examineExampleStrange(int);

  int examineExampleNonBound(int);
  int secondChoiceNonBound(int);

  double lambda;

  void mainSMO();
  matrix K;
  matrix x;
  seq y;
	
  seq alpha;
  double beta;
	
  int secondChoice(int i);
	
  double f(int);
  double fLambda(int);
	
  void updateBeta();

  void createSets(set & , set & , set &);
	
  int numNonZeroBound();


  void buildNonZeroBound();
  list<int> nonBoundSV;
        

  bool violates(int i);
  bool violatesLambda(int i);
	
  void simpleSMO();

  bool notBound(double d);

  int nMinus, nPlus, minor, nMinor;

  SMO(matrix & xPar, seq & yPar);

  vector<double> kDiag;
	
  inline double kernel(int k, int j)
    {
      if (CACHE)
	return kernelMat[k][j];
      if (k==j)
	{
	  return kDiag[k];
	}
      return kFunction(x[k], x[j]);
    }

  void verify();

  vector<double> fC;

  double fCache(int p);

  int cycle;

  void printModel();
};

#endif
