/*******
svmvia, an implementation of the full regularization path for support vector machines
Copyright (C) 2007 Oliver Serang
Written by Oliver Serang at the Department of Genome Sciences at the University of Washington

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*********/

#ifndef SVM_H_
#define SVM_H_

#include <iostream>
#include <fstream>
#include "set.h"
#include "seq.h"
#include "matrix.h"
#include "basic.h"
#include "smo.h"
#include <algorithm>

using namespace std;

class SVM
{
 public:
  SVM() {}

  SVM(const matrix & xArg, const seq & yArg, const matrix & vX, const seq & vY);

  int max_cycles;

  double lambdaOpt, alpha0Opt;
  seq alphaOpt;

  double bestError;

  matrix kStar;

  int valN;
  int cycle;

  seq fValL, gValL, dValL;

  matrix x;
  seq y;

  matrix valX;
  seq valY;

  int n, nPlus, nMinus;
  int nPlusVal, nMinusVal;

  set left, right, elbow;
  set all;

  seq fL, gL, dL;
  double kernel(int , int);
  seq alpha;
  double alpha0;

  seq kDiag;

  double lambda;

  void computeInitialLargeF();
  seq fLargeInit;

  void addLeft(int);
  void removeLeft(int);
  void removeLeft(const set & s);

  seq largeFInit(const set & s);
  seq getF();
  
  void fitPath();
  void balancedInit(const set & s);
  void unbalancedInit();

  void updateKStar(const set & rhs);
  void updateKStarOther(int i);

  void downdateKStar(int i);

  seq solveKStar();

  double deNovo(const seq & xA);
  double deNovoOpt(const seq & xA);

  double error();
  double equalWeightError();
  double weightError();
  double rocError();

  double validate(const matrix & xPoints, const seq & yPoints);

  void updateOptima();
  seq getValF();
};

#endif

